@@ -74,11 +74,13 @@ extern crate tracing;
 pub mod base64_data;
 pub mod crypto;
 pub mod error;
+pub mod jws;
 pub mod jwt;
 pub mod oidc;
 
 pub use crate::crypto::{JwaAlg, Jwk, JwkKeySet, JwkUse, JwsSigner, JwsValidator};
 pub use crate::error::JwtError;
+pub use crate::jws::{Jws, JwsSigned, JwsUnverified};
 pub use crate::jwt::{Jwt, JwtSigned, JwtUnverified};
 pub use crate::oidc::{OidcClaims, OidcSigned, OidcSubject, OidcToken, OidcUnverified};
 
@@ -88,6 +90,6 @@ pub(crate) fn btreemap_empty(
     m.is_empty()
 }
 
-pub(crate) fn vec_empty(m: &Vec<String>) -> bool {
+pub(crate) fn vec_empty(m: &[String]) -> bool {
     m.is_empty()
 }
